/*##############################################################################

Copyright (C) 2007 Ulrich Radig

##############################################################################*/

#include "usart.h"

volatile BOOL usart_ready;
volatile unsigned char usart_buffer[BUFFER_SIZE];
volatile unsigned char usart_get[BUFFER_SIZE];

static unsigned char buffer_index;
static unsigned char receive_char;

//##############################################################################
//Init serielle Schnittstelle
void usart_init(void) 
//##############################################################################
{ 
  	//Enable TXEN im Register UCR TX-Data Enable
	UCR=(1 << TXEN)|(1 << RXEN)|(1<<RXCIE);
	//Teiler wird gesetzt 
	UBRR=(SYSCLK / (BAUD_RATE * 16L) - 1);
	//ffnet einen Kanal fr printf (STDOUT)
	//fdevopen (usart_write_char, NULL);
}

//##############################################################################
//Routine fr die Serielle Ausgabe
int usart_write_char(char c)
//##############################################################################
{
	if (c == '\n')
		usart_write_char('\r');
	//Warten solange bis Zeichen gesendet wurde
	while(!(USR & (1<<UDRE)));
	//Ausgabe des Zeichens
	UDR = c;
	return(0);
}

//##############################################################################
void usart_writeln_flash_str(const char *str)
//##############################################################################
{

  while (*str)
  {
    usart_write_char(*str++);
  }
  //usart_write_char(CR);
  //usart_write_char(LF);
}

//##############################################################################
void usart_write_str(char *str)
//##############################################################################
{

  while (*str)
  {
    usart_write_char(*str++);
  }
}

//##############################################################################
void usart_writeln_str(char *data)
//##############################################################################
{
  usart_write_str(data);
  usart_write_char(CR);
  usart_write_char(LF);
}

//##############################################################################
void usart_write_U08(char data) // Byte ausgeben
//##############################################################################
{
  usart_write_str((char *)U08_to_str(data));
}

//##############################################################################
void usart_write_U16(int data) // Byte ausgeben
//##############################################################################
{
  usart_write_str((char *)U16_to_str(data));
}

//##############################################################################
void usart_write_U08_hex(char data) // Byte hexadezimal ausgeben
//##############################################################################
{
  usart_write_str((char *)U08_to_hex(data));
}

//##############################################################################
SIGNAL (RX_INTERRUPT)
//##############################################################################
{
  receive_char = UDR;

  if (receive_char == CR && usart_ready == 0) // Wenn Zeichen CR ist, dann Ready-Flag setzen
  {
    usart_buffer[buffer_index] = 0; // Stringterminierung
    buffer_index = 0;
    usart_ready = TRUE;
    return;    
  }

  if (buffer_index < BUFFER_SIZE - 1) // Empfangszeichen speichern
  {
    usart_buffer[buffer_index++] = receive_char;    
  }
return;
}


